@@ -97,10 +97,15 @@ class Eaves:
 
     def handler(self, name: str, func_before_running: Optional[Callable[[list[Event]], Any]] = None) -> Callable:
         def inner(func: Callable[[list[Event]], Any]) -> Callable:
-            if func_before_running:
-                func = lambda event: func(func_before_running(event)) # type: ignore
 
-            self.handlers[name] = func
+            if func_before_running:
+                ret = func.__annotations__['return']
+                def new_func(events) -> ret:
+                    func(func_before_running(events))
+
+                self.handlers[name] = new_func
+            else:
+                self.handlers[name] = func
             return func
         return inner
 
